from setuptools import setup, find_packages

setup(name="ZeroMqWsgiWrapper",
      version="1.0.0",
      packages=find_packages(where="."),
      py_modules=["main_flask"],
      install_requires=['EdpHelper==0.14',
                        'Flask==1.1.1',
                        'ZeroMqMessageHandler==1.0.0'],
      url="http://www.e-deploy.com.br",
      author="Eduardo Almeida",
      author_email="eduardo.almeida@e-deploy.com.br")
